@@ -550,7 +550,7 @@ class Item_func_seconds_hybrid: public Item_func_numhybrid
 public:
   Item_func_seconds_hybrid(THD *thd): Item_func_numhybrid(thd) {}
   Item_func_seconds_hybrid(THD *thd, Item *a): Item_func_numhybrid(thd, a) {}
-  void fix_length_and_dec_generic(uint dec)
+  void fix_length_and_dec_generic(decimal_digits_t dec)
   {
     DBUG_ASSERT(dec <= TIME_SECOND_PART_DIGITS);
     decimals= dec;
@@ -725,7 +725,8 @@ class Item_func_curtime :public Item_timefunc
   MYSQL_TIME ltime;
   query_id_t last_query_id;
 public:
-  Item_func_curtime(THD *thd, uint dec): Item_timefunc(thd), last_query_id(0)
+  Item_func_curtime(THD *thd, decimal_digits_t dec):
+    Item_timefunc(thd), last_query_id(0)
   { decimals= dec; }
   bool fix_fields(THD *, Item **) override;
   bool fix_length_and_dec(THD *thd) override
@@ -748,7 +749,8 @@ public:
 class Item_func_curtime_local :public Item_func_curtime
 {
 public:
-  Item_func_curtime_local(THD *thd, uint dec): Item_func_curtime(thd, dec) {}
+  Item_func_curtime_local(THD *thd, decimal_digits_t dec):
+    Item_func_curtime(thd, dec) {}
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("curtime") };
@@ -763,7 +765,8 @@ public:
 class Item_func_curtime_utc :public Item_func_curtime
 {
 public:
-  Item_func_curtime_utc(THD *thd, uint dec): Item_func_curtime(thd, dec) {}
+  Item_func_curtime_utc(THD *thd, decimal_digits_t dec):
+    Item_func_curtime(thd, dec) {}
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("utc_time") };
@@ -829,7 +832,8 @@ class Item_func_now :public Item_datetimefunc
   MYSQL_TIME ltime;
   query_id_t last_query_id;
 public:
-  Item_func_now(THD *thd, uint dec): Item_datetimefunc(thd), last_query_id(0)
+  Item_func_now(THD *thd, decimal_digits_t dec):
+    Item_datetimefunc(thd), last_query_id(0)
   { decimals= dec; }
   bool fix_fields(THD *, Item **) override;
   bool fix_length_and_dec(THD *thd) override
@@ -851,7 +855,8 @@ public:
 class Item_func_now_local :public Item_func_now
 {
 public:
-  Item_func_now_local(THD *thd, uint dec): Item_func_now(thd, dec) {}
+  Item_func_now_local(THD *thd, decimal_digits_t dec):
+    Item_func_now(thd, dec) {}
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("current_timestamp") };
@@ -868,7 +873,7 @@ public:
 class Item_func_now_utc :public Item_func_now
 {
 public:
-  Item_func_now_utc(THD *thd, uint dec): Item_func_now(thd, dec) {}
+  Item_func_now_utc(THD *thd, decimal_digits_t dec): Item_func_now(thd, dec) {}
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("utc_timestamp") };
@@ -893,7 +898,8 @@ public:
 class Item_func_sysdate_local :public Item_func_now
 {
 public:
-  Item_func_sysdate_local(THD *thd, uint dec): Item_func_now(thd, dec) {}
+  Item_func_sysdate_local(THD *thd, decimal_digits_t dec):
+    Item_func_now(thd, dec) {}
   bool const_item() const override { return 0; }
   LEX_CSTRING func_name_cstring() const override
   {
@@ -962,7 +968,7 @@ public:
   }
   bool fix_length_and_dec(THD *thd) override;
   uint format_length(const String *format);
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq(const Item *item, const Eq_config &config) const override;
   bool check_vcol_func_processor(void *arg) override
   {
     if (arg_count > 2)
@@ -1154,7 +1160,7 @@ public:
     return name;
   }
   bool fix_length_and_dec(THD *thd) override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq(const Item *item, const Eq_config &config) const override;
   void print(String *str, enum_query_type query_type) override;
   enum precedence precedence() const override { return INTERVAL_PRECEDENCE; }
   bool need_parentheses_in_default() override { return true; }
@@ -1225,7 +1231,7 @@ class Item_extract :public Item_int_func,
   }
   bool check_arguments() const override;
   bool fix_length_and_dec(THD *thd) override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq(const Item *item, const Eq_config &config) const override;
   void print(String *str, enum_query_type query_type) override;
   bool check_partition_func_processor(void *int_arg) override {return FALSE;}
   bool check_vcol_func_processor(void *arg) override
@@ -1298,7 +1304,7 @@ public:
     Item_handled_func(thd, a), cast_length(length_arg), cast_cs(cs_arg),
     m_suppress_warning_to_error_escalation(false) {}
   enum Functype functype() const override { return CHAR_TYPECAST_FUNC; }
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq(const Item *item, const Eq_config &config) const override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("cast_as_char") };
@@ -1367,7 +1373,7 @@ public:
 class Item_time_typecast :public Item_timefunc
 {
 public:
-  Item_time_typecast(THD *thd, Item *a, uint dec_arg):
+  Item_time_typecast(THD *thd, Item *a, decimal_digits_t dec_arg):
     Item_timefunc(thd, a) { decimals= dec_arg; }
   LEX_CSTRING func_name_cstring() const override
   {
@@ -1393,7 +1399,7 @@ public:
 class Item_datetime_typecast :public Item_datetimefunc
 {
 public:
-  Item_datetime_typecast(THD *thd, Item *a, uint dec_arg):
+  Item_datetime_typecast(THD *thd, Item *a, decimal_digits_t dec_arg):
     Item_datetimefunc(thd, a) { decimals= dec_arg; }
   LEX_CSTRING func_name_cstring() const override
   {
@@ -1452,8 +1458,8 @@ public:
   }
   bool fix_length_and_dec(THD *thd) override
   {
-    uint dec0= args[0]->datetime_precision(thd);
-    uint dec1= Interval_DDhhmmssff::fsp(thd, args[1]);
+    decimal_digits_t dec0= args[0]->datetime_precision(thd);
+    decimal_digits_t dec1= Interval_DDhhmmssff::fsp(thd, args[1]);
     fix_attributes_datetime(MY_MAX(dec0, dec1));
     set_maybe_null();
     return false;
@@ -1520,8 +1526,8 @@ public:
   }
   bool fix_length_and_dec(THD *thd) override
   {
-    uint dec= MY_MAX(args[0]->time_precision(thd),
-                     args[1]->time_precision(thd));
+    decimal_digits_t dec= MY_MAX(args[0]->time_precision(thd),
+                                 args[1]->time_precision(thd));
     fix_attributes_time(dec);
     set_maybe_null();
     return FALSE;
@@ -1693,7 +1699,8 @@ public:
 class Func_handler_date_add_interval
 {
 protected:
-  static uint interval_dec(const Item *item, interval_type int_type)
+  static decimal_digits_t interval_dec(const Item *item,
+                                       interval_type int_type)
   {
     if (int_type == INTERVAL_MICROSECOND ||
         (int_type >= INTERVAL_DAY_MICROSECOND &&
@@ -1730,8 +1737,9 @@ class Func_handler_date_add_interval_datetime:
 public:
   bool fix_length_and_dec(Item_handled_func *item) const override
   {
-    uint dec= MY_MAX(item->arguments()[0]->datetime_precision(current_thd),
-                     interval_dec(item->arguments()[1], int_type(item)));
+    decimal_digits_t dec=
+      MY_MAX(item->arguments()[0]->datetime_precision(current_thd),
+             interval_dec(item->arguments()[1], int_type(item)));
     item->fix_attributes_datetime(dec);
     return false;
   }
@@ -1789,8 +1797,9 @@ class Func_handler_date_add_interval_time:
 public:
   bool fix_length_and_dec(Item_handled_func *item) const override
   {
-    uint dec= MY_MAX(item->arguments()[0]->time_precision(current_thd),
-                     interval_dec(item->arguments()[1], int_type(item)));
+    decimal_digits_t dec=
+      MY_MAX(item->arguments()[0]->time_precision(current_thd),
+             interval_dec(item->arguments()[1], int_type(item)));
     item->fix_attributes_time(dec);
     return false;
   }
@@ -1814,8 +1823,9 @@ class Func_handler_date_add_interval_string:
 public:
   bool fix_length_and_dec(Item_handled_func *item) const override
   {
-    uint dec= MY_MAX(item->arguments()[0]->datetime_precision(current_thd),
-                     interval_dec(item->arguments()[1], int_type(item)));
+    decimal_digits_t dec=
+      MY_MAX(item->arguments()[0]->datetime_precision(current_thd),
+             interval_dec(item->arguments()[1], int_type(item)));
     item->Type_std_attributes::set(
       Type_temporal_attributes_not_fixed_dec(MAX_DATETIME_WIDTH, dec, false),
       DTCollation(item->default_charset(),
@@ -1856,8 +1866,8 @@ public:
   bool fix_length_and_dec(Item_handled_func *item) const override
   {
     THD *thd= current_thd;
-    uint dec0= item->arguments()[0]->datetime_precision(thd);
-    uint dec1= Interval_DDhhmmssff::fsp(thd, item->arguments()[1]);
+    decimal_digits_t dec0= item->arguments()[0]->datetime_precision(thd);
+    decimal_digits_t dec1= Interval_DDhhmmssff::fsp(thd, item->arguments()[1]);
     item->fix_attributes_datetime(MY_MAX(dec0, dec1));
     return false;
   }
@@ -1890,8 +1900,8 @@ public:
   bool fix_length_and_dec(Item_handled_func *item) const override
   {
     THD *thd= current_thd;
-    uint dec0= item->arguments()[0]->time_precision(thd);
-    uint dec1= Interval_DDhhmmssff::fsp(thd, item->arguments()[1]);
+    decimal_digits_t dec0= item->arguments()[0]->time_precision(thd);
+    decimal_digits_t dec1= Interval_DDhhmmssff::fsp(thd, item->arguments()[1]);
     item->fix_attributes_time(MY_MAX(dec0, dec1));
     return false;
   }
@@ -1922,9 +1932,10 @@ public:
   { }
   bool fix_length_and_dec(Item_handled_func *item) const override
   {
-    uint dec0= item->arguments()[0]->decimals;
-    uint dec1= Interval_DDhhmmssff::fsp(current_thd, item->arguments()[1]);
-    uint dec= MY_MAX(dec0, dec1);
+    decimal_digits_t dec0= item->arguments()[0]->decimals;
+    decimal_digits_t dec1=
+      Interval_DDhhmmssff::fsp(current_thd, item->arguments()[1]);
+    decimal_digits_t dec= MY_MAX(dec0, dec1);
     item->Type_std_attributes::set(
       Type_temporal_attributes_not_fixed_dec(MAX_DATETIME_WIDTH, dec, false),
       DTCollation(item->default_charset(),
