@@ -601,7 +601,7 @@ class binlog_cache_data;
 struct rpl_gtid;
 struct wait_for_commit;
 
-class MYSQL_BIN_LOG: public TC_LOG, private Event_log
+class MYSQL_BIN_LOG: public TC_LOG, public Event_log
 {
 #ifdef HAVE_PSI_INTERFACE
   /** The instrumentation key to use for @ LOCK_index. */
@@ -1013,7 +1013,7 @@ public:
   bool write_incident_already_locked(THD *thd);
   bool write_incident(THD *thd);
   void write_binlog_checkpoint_event_already_locked(const char *name, uint len);
-  bool write_table_map(THD *thd, TABLE *table, bool with_annotate);
+  bool write_table_map(THD *thd, TABLE *table);
 
   void start_union_events(THD *thd, query_id_t query_id_param);
   void stop_union_events(THD *thd);
@@ -1211,6 +1211,7 @@ public:
 };
 
 
+int check_if_log_table(const TABLE_LIST *table);
 int check_if_log_table(const TABLE_LIST *table, bool check_if_opened,
                        const char *errmsg);
 
@@ -1384,6 +1385,7 @@ File open_binlog(IO_CACHE *log, const char *log_file_name,
 
 void make_default_log_name(char **out, const char* log_ext, bool once);
 void binlog_reset_cache(THD *thd);
+void binlog_clear_incident(THD *thd);
 bool write_annotated_row(THD *thd);
 int binlog_flush_pending_rows_event(THD *thd, bool stmt_end,
                                     bool is_transactional,
@@ -1430,8 +1432,8 @@ inline bool normalize_binlog_name(char *to, const char *from, bool is_relay_log)
   /* opt_name is not null and not empty and from is a relative path */
   if (opt_name && opt_name[0] && from && !test_if_hard_path(from))
   {
-    // take the path from opt_name
-    // take the filename from from 
+    // take the path from "opt_name"
+    // take the filename from "from"
     char log_dirpart[FN_REFLEN], log_dirname[FN_REFLEN];
     size_t log_dirpart_len, log_dirname_len;
     dirname_part(log_dirpart, opt_name, &log_dirpart_len);
