@@ -1,7 +1,7 @@
 #ifndef MDL_H
 #define MDL_H
 /* Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
-   Copyright (c) 2020, MariaDB
+   Copyright (c) 2020, 2021, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #include <m_string.h>
 #include <mysql_com.h>
 #include <lf.h>
+#include "lex_ident.h"
 
 class THD;
 
 class MDL_context;
 class MDL_lock;
 class MDL_ticket;
-bool  ok_for_lower_case_names(const char *name);
 
 typedef unsigned short mdl_bitmap_t;
 #define MDL_BIT(A) static_cast<mdl_bitmap_t>(1U << A)
@@ -60,7 +60,7 @@ typedef unsigned short mdl_bitmap_t;
 class MDL_context_owner
 {
 public:
-  virtual ~MDL_context_owner() {}
+  virtual ~MDL_context_owner() = default;
 
   /**
     Enter a condition wait.
@@ -172,7 +172,7 @@ enum enum_mdl_type {
     cases when we only need to access metadata and not data, e.g. when
     filling an INFORMATION_SCHEMA table.
     Since SH lock is compatible with SNRW lock, the connection that
-    holds SH lock lock should not try to acquire any kind of table-level
+    holds the SH lock should not try to acquire any kind of table-level
     or row-level lock, as this can lead to a deadlock. Moreover, after
     acquiring SH lock, the connection should not wait for any other
     resource, as it might cause starvation for X locks and a potential
@@ -414,8 +414,8 @@ public:
 
     @param  mdl_namespace Id of namespace of object to be locked
     @param  db            Name of database to which the object belongs
-    @param  name          Name of of the object
-    @param  key           Where to store the the MDL key.
+    @param  name          Name of the object
+    @param  key           Where to store the MDL key.
   */
   void mdl_key_init(enum_mdl_namespace mdl_namespace_arg,
                     const char *db, const char *name_arg)
@@ -435,7 +435,9 @@ public:
                                           NAME_LEN) - m_ptr + 1);
     m_hash_value= my_hash_sort(&my_charset_bin, (uchar*) m_ptr + 1,
                                m_length - 1);
-    DBUG_SLOW_ASSERT(mdl_namespace_arg == USER_LOCK || ok_for_lower_case_names(db));
+    DBUG_SLOW_ASSERT(mdl_namespace_arg == USER_LOCK ||
+                     Lex_ident_fs(db, m_db_name_length).
+                       ok_for_lower_case_names());
   }
   void mdl_key_init(const MDL_key *rhs)
   {
@@ -471,7 +473,7 @@ public:
   {
     mdl_key_init(namespace_arg, db_arg, name_arg);
   }
-  MDL_key() {} /* To use when part of MDL_request. */
+  MDL_key() = default; /* To use when part of MDL_request. */
 
   /**
     Get thread state name to be used in case when we have to
@@ -640,7 +642,7 @@ public:
 
   virtual bool inspect_edge(MDL_context *dest) = 0;
   virtual ~MDL_wait_for_graph_visitor();
-  MDL_wait_for_graph_visitor() {}
+  MDL_wait_for_graph_visitor() = default;
 };
 
 /**
@@ -699,7 +701,17 @@ public:
   */
   MDL_ticket *next_in_context;
   MDL_ticket **prev_in_context;
+
+#ifndef DBUG_OFF
+  /**
+    Duration of lock represented by this ticket.
+    Context public. Debug-only.
+  */
 public:
+  enum_mdl_duration m_duration;
+#endif
+  ulonglong m_time;
+
 #ifdef WITH_WSREP
   void wsrep_report(bool debug) const;
 #endif /* WITH_WSREP */
@@ -726,8 +738,8 @@ public:
   bool is_incompatible_when_waiting(enum_mdl_type type) const;
 
   /** Implement MDL_wait_for_subgraph interface. */
-  virtual bool accept_visitor(MDL_wait_for_graph_visitor *dvisitor);
-  virtual uint get_deadlock_weight() const;
+  bool accept_visitor(MDL_wait_for_graph_visitor *dvisitor) override;
+  uint get_deadlock_weight() const override;
   /**
     Status of lock request represented by the ticket as reflected in P_S.
   */
@@ -741,10 +753,12 @@ private:
              , enum_mdl_duration duration_arg
 #endif
             )
-   : m_type(type_arg),
+   :
 #ifndef DBUG_OFF
      m_duration(duration_arg),
 #endif
+     m_time(0),
+     m_type(type_arg),
      m_ctx(ctx_arg),
      m_lock(NULL),
      m_psi(NULL)
@@ -764,13 +778,7 @@ private:
 private:
   /** Type of metadata lock. Externally accessible. */
   enum enum_mdl_type m_type;
-#ifndef DBUG_OFF
-  /**
-    Duration of lock represented by this ticket.
-    Context private. Debug-only.
-  */
-  enum_mdl_duration m_duration;
-#endif
+
   /**
     Context of the owner of the metadata lock ticket. Externally accessible.
   */
@@ -799,7 +807,7 @@ private:
 class MDL_savepoint
 {
 public:
-  MDL_savepoint() {};
+  MDL_savepoint() = default;;
 
 private:
   MDL_savepoint(MDL_ticket *stmt_ticket, MDL_ticket *trans_ticket)
@@ -905,6 +913,10 @@ public:
              m_tickets[MDL_TRANSACTION].is_empty() &&
              m_tickets[MDL_EXPLICIT].is_empty());
   }
+  bool has_explicit_locks() const
+  {
+    return !m_tickets[MDL_EXPLICIT].is_empty();
+  }
   inline bool has_transactional_locks() const
   {
     return !m_tickets[MDL_TRANSACTION].is_empty();
@@ -939,7 +951,8 @@ public:
                     already has received some signal or closed
                     signal slot.
   */
-  void init(MDL_context_owner *arg) { m_owner= arg; }
+  void init(MDL_context_owner *arg) { m_owner= arg; reset(); }
+  void reset() { m_deadlock_overweight= 0; }
 
   void set_needs_thr_lock_abort(bool needs_thr_lock_abort)
   {
@@ -1048,7 +1061,7 @@ private:
    */
   MDL_wait_for_subgraph *m_waiting_for;
   LF_PINS *m_pins;
-  uint m_deadlock_overweight= 0;
+  uint m_deadlock_overweight;
 private:
   MDL_ticket *find_ticket(MDL_request *mdl_req,
                           enum_mdl_duration *duration);
@@ -1096,6 +1109,26 @@ private:
 
   /* metadata_lock_info plugin */
   friend int i_s_metadata_lock_info_fill_row(MDL_ticket*, void*);
+#ifndef DBUG_OFF
+public:
+  /**
+    This is for the case when the thread opening the table does not acquire
+    the lock itself, but utilizes a lock guarantee from another MDL context.
+
+    For example, in InnoDB, MDL is acquired by the purge_coordinator_task,
+    but the table may be opened and used in a purge_worker_task.
+    The coordinator thread holds the lock for the duration of worker's purge
+    job, or longer, possibly reusing shared MDL for different workers and jobs.
+  */
+  MDL_context *lock_warrant= NULL;
+
+  inline bool is_lock_warrantee(MDL_key::enum_mdl_namespace ns,
+                                const char *db, const char *name,
+                                enum_mdl_type mdl_type) const
+  {
+    return lock_warrant && lock_warrant->is_lock_owner(ns, db, name, mdl_type);
+  }
+#endif
 };
 
 
@@ -1126,9 +1159,4 @@ typedef int (*mdl_iterator_callback)(MDL_ticket *ticket, void *arg,
                                      bool granted);
 extern MYSQL_PLUGIN_IMPORT
 int mdl_iterate(mdl_iterator_callback callback, void *arg);
-#ifndef DBUG_OFF
-void mdl_dbug_print_locks();
-#else
-  static inline void mdl_dbug_print_locks() {}
-#endif /* DBUG_OFF */
 #endif /* MDL_H */
