@@ -72,14 +72,18 @@ struct Opt_trace_info
 */
 
 
-class Opt_trace_start {
+class Opt_trace_start
+{
  public:
-  Opt_trace_start(THD *thd_arg, TABLE_LIST *tbl,
-                  enum enum_sql_command sql_command,
-                  List<set_var_base> *set_vars,
-                  const char *query,
-                  size_t query_length,
-                  const CHARSET_INFO *query_charset);
+  Opt_trace_start(THD *thd_arg): ctx(&thd_arg->opt_trace), traceable(false) {}
+
+  void init(THD *thd, TABLE_LIST *tbl,
+            enum enum_sql_command sql_command,
+            List<set_var_base> *set_vars,
+            const char *query,
+            size_t query_length,
+            const CHARSET_INFO *query_charset);
+
   ~Opt_trace_start();
 
  private:
@@ -103,10 +107,10 @@ void opt_trace_print_expanded_query(THD *thd, SELECT_LEX *select_lex,
                                     Json_writer_object *trace_object);
 
 void add_table_scan_values_to_trace(THD *thd, JOIN_TAB *tab);
-void trace_plan_prefix(JOIN *join, uint idx, table_map join_tables);
+void trace_plan_prefix(Json_writer_object *jsobj, JOIN *join, uint idx,
+                       table_map join_tables);
 void print_final_join_order(JOIN *join);
-void print_best_access_for_table(THD *thd, POSITION *pos,
-                                 enum join_type type);
+void print_best_access_for_table(THD *thd, POSITION *pos);
 
 void trace_condition(THD * thd, const char *name, const char *transform_type,
                     Item *item, const char *table_name= nullptr);
