@@ -84,6 +84,7 @@ class Explain_node : public Sql_alloc
 public:
   Explain_node(MEM_ROOT *root) :
     cache_tracker(NULL),
+    subq_materialization(NULL),
     connection_type(EXPLAIN_NODE_OTHER),
     children(root)
   {}
@@ -113,6 +114,12 @@ public:
   */
   Expression_cache_tracker* cache_tracker;
 
+  /**
+    If not NULL, this node is a SELECT (or UNION) in a materialized
+    IN-subquery.
+  */
+  Explain_subq_materialization* subq_materialization;
+
   /*
     How this node is connected to its parent.
     (NOTE: EXPLAIN_NODE_NON_MERGED_SJ is set very late currently)
@@ -134,14 +141,15 @@ public:
   virtual int print_explain(Explain_query *query, select_result_sink *output, 
                             uint8 explain_flags, bool is_analyze)=0;
   virtual void print_explain_json(Explain_query *query, Json_writer *writer, 
-                                  bool is_analyze, bool no_tmp_tbl)= 0;
+                                  bool is_analyze)= 0;
 
   int print_explain_for_children(Explain_query *query, select_result_sink *output, 
                                  uint8 explain_flags, bool is_analyze);
   void print_explain_json_for_children(Explain_query *query,
-                                       Json_writer *writer, bool is_analyze,
-                                       bool no_tmp_tbl);
+                                       Json_writer *writer, bool is_analyze);
   bool print_explain_json_cache(Json_writer *writer, bool is_analyze);
+  bool print_explain_json_subq_materialization(Json_writer *writer,
+                                               bool is_analyze);
   virtual ~Explain_node() = default;
 };
 
@@ -160,24 +168,24 @@ class Explain_table_access;
 class Explain_basic_join : public Explain_node
 {
 public:
-  enum explain_node_type get_type() { return EXPLAIN_BASIC_JOIN; }
+  enum explain_node_type get_type() override { return EXPLAIN_BASIC_JOIN; }
   
   Explain_basic_join(MEM_ROOT *root) : Explain_node(root), join_tabs(NULL) {}
   ~Explain_basic_join();
 
   bool add_table(Explain_table_access *tab, Explain_query *query);
 
-  uint get_select_id() { return select_id; }
+  uint get_select_id() override { return select_id; }
 
   uint select_id;
 
   int print_explain(Explain_query *query, select_result_sink *output,
-                    uint8 explain_flags, bool is_analyze);
+                    uint8 explain_flags, bool is_analyze) override;
   void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze, bool no_tmp_tbl);
+                          bool is_analyze) override;
 
   void print_explain_json_interns(Explain_query *query, Json_writer *writer,
-                                  bool is_analyze, bool no_tmp_tbl);
+                                  bool is_analyze);
 
   /* A flat array of Explain structs for tables. */
   Explain_table_access** join_tabs;
@@ -204,7 +212,7 @@ class Explain_aggr_node;
 class Explain_select : public Explain_basic_join
 {
 public:
-  enum explain_node_type get_type() { return EXPLAIN_SELECT; }
+  enum explain_node_type get_type() override { return EXPLAIN_SELECT; }
 
   Explain_select(MEM_ROOT *root, bool is_analyze) : 
   Explain_basic_join(root),
@@ -216,6 +224,7 @@ public:
     message(NULL),
     having(NULL), having_value(Item::COND_UNDEF),
     using_temporary(false), using_filesort(false),
+    cost(0.0),
     time_tracker(is_analyze),
     aggr_tree(NULL)
   {}
@@ -249,9 +258,10 @@ public:
   bool using_temporary;
   bool using_filesort;
 
+  double cost;
   /* ANALYZE members */
   Time_and_counter_tracker time_tracker;
-  
+
   /* 
     Part of query plan describing sorting, temp.table usage, and duplicate 
     removal
@@ -259,9 +269,9 @@ public:
   Explain_aggr_node* aggr_tree;
 
   int print_explain(Explain_query *query, select_result_sink *output, 
-                    uint8 explain_flags, bool is_analyze);
+                    uint8 explain_flags, bool is_analyze) override;
   void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze, bool no_tmp_tbl);
+                          bool is_analyze) override;
   
   Table_access_tracker *get_using_temporary_read_tracker()
   {
@@ -298,42 +308,41 @@ class Explain_aggr_filesort : public Explain_aggr_node
   List<Item> sort_items;
   List<ORDER::enum_order> sort_directions;
 public:
-  enum_explain_aggr_node_type get_type() { return AGGR_OP_FILESORT; }
+  enum_explain_aggr_node_type get_type() override { return AGGR_OP_FILESORT; }
   Filesort_tracker tracker;
 
   Explain_aggr_filesort(MEM_ROOT *mem_root, bool is_analyze, 
                         Filesort *filesort);
 
-  void print_json_members(Json_writer *writer, bool is_analyze,
-                          bool no_tmp_tbl);
+  void print_json_members(Json_writer *writer, bool is_analyze);
 };
 
 class Explain_aggr_tmp_table : public Explain_aggr_node
 {
 public:
-  enum_explain_aggr_node_type get_type() { return AGGR_OP_TEMP_TABLE; }
+  enum_explain_aggr_node_type get_type() override { return AGGR_OP_TEMP_TABLE; }
 };
 
 class Explain_aggr_remove_dups : public Explain_aggr_node
 {
 public:
-  enum_explain_aggr_node_type get_type() { return AGGR_OP_REMOVE_DUPLICATES; }
+  enum_explain_aggr_node_type get_type() override { return AGGR_OP_REMOVE_DUPLICATES; }
 };
 
 class Explain_aggr_window_funcs : public Explain_aggr_node
 {
   List<Explain_aggr_filesort> sorts;
 public:
-  enum_explain_aggr_node_type get_type() { return AGGR_OP_WINDOW_FUNCS; }
+  enum_explain_aggr_node_type get_type() override { return AGGR_OP_WINDOW_FUNCS; }
 
-  void print_json_members(Json_writer *writer, bool is_analyze,
-                          bool no_tmp_tbl);
+  void print_json_members(Json_writer *writer, bool is_analyze);
   friend class Window_funcs_computation;
 };
 
 /////////////////////////////////////////////////////////////////////////////
 
 extern const char *unit_operation_text[4];
+extern const char *pushed_unit_operation_text[4];
 extern const char *pushed_derived_text;
 extern const char *pushed_select_text;
 
@@ -348,14 +357,14 @@ class Explain_union : public Explain_node
 public:
   Explain_union(MEM_ROOT *root, bool is_analyze) : 
     Explain_node(root), union_members(PSI_INSTRUMENT_MEM),
-    is_recursive_cte(false),
+    is_recursive_cte(false), is_pushed_down_to_engine(false),
     fake_select_lex_explain(root, is_analyze)
   {}
 
-  enum explain_node_type get_type() { return EXPLAIN_UNION; }
+  enum explain_node_type get_type() override { return EXPLAIN_UNION; }
   unit_common_op operation;
 
-  uint get_select_id()
+  uint get_select_id() override
   {
     DBUG_ASSERT(union_members.elements() > 0);
     return union_members.at(0);
@@ -377,15 +386,20 @@ public:
   {
     union_members.append(select_no);
   }
-  int print_explain(Explain_query *query, select_result_sink *output, 
-                    uint8 explain_flags, bool is_analyze);
-  void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze, bool no_tmp_tbl);
+  int print_explain(Explain_query *query, select_result_sink *output,
+                    uint8 explain_flags, bool is_analyze) override;
+  void print_explain_json(Explain_query *query, Json_writer *writer,
+                          bool is_analyze) override;
+  void print_explain_json_regular(Explain_query *query, Json_writer *writer,
+                          bool is_analyze);
+  void print_explain_json_pushed_down(Explain_query *query,
+                                      Json_writer *writer, bool is_analyze);
 
   const char *fake_select_type;
   bool using_filesort;
   bool using_tmp;
   bool is_recursive_cte;
+  bool is_pushed_down_to_engine;
   
   /*
     Explain data structure for "fake_select_lex" (i.e. for the degenerate
@@ -404,6 +418,10 @@ public:
   }
 private:
   uint make_union_table_name(char *buf);
+  int print_explain_regular(Explain_query *query, select_result_sink *output,
+                            uint8 explain_flags, bool is_analyze);
+  int print_explain_pushed_down(select_result_sink *output,
+                                uint8 explain_flags, bool is_analyze);
   
   Table_access_tracker fake_select_lex_tracker;
   /* This one is for reading after ORDER BY */
@@ -448,19 +466,19 @@ class Explain_insert;
 
   (1) - Query plan construction is finished and it is available for reading.
 
-  (2) - Temporary tables are freed. After this point,
-        we need to pass QT_DONT_ACCESS_TMP_TABLES to item->print(). Since
-        we don't track when #2 happens for each temp.table, we pass this
-        flag whenever we're printing the query plan for a SHOW command.
-        Also, we pass it when printing ANALYZE (?)
+  (2) - Temporary tables are freed (with exception of derived tables
+        which are freed at step (4)).
+        The tables are no longer accessible but one can still call
+        item->print(), even for items that refer to temp.tables (see
+        Item_field::print() for details)
 
   (3) - Notification about (4).
-  (4) - Tables used by the query are closed. One known consequence of this is
-        that the values of the const tables' fields are not available anymore.
-        We could use the same approach as in QT_DONT_ACCESS_TMP_TABLES to work
-        around that, but instead we disallow producing FORMAT=JSON output at
-        step #3. We also processing of SHOW command. The rationale is that
-        query is close to finish anyway.
+  (4) - Tables used by the query are closed. One consequence of this is that
+        the values of the const tables' fields are not available anymore.
+        We could adjust the code in Item_field::print() to handle this but
+        instead we make step (3) disallow production of FORMAT=JSON output.
+        We also disable processing of SHOW EXPLAIN|ANALYZE output because
+        the query is about to finish anyway.
 
   (5) - Item objects are freed. After this, it's certainly not possible to
         print them into FORMAT=JSON output.
@@ -499,7 +517,6 @@ public:
   bool print_explain_str(THD *thd, String *out_str, bool is_analyze);
 
   int print_explain_json(select_result_sink *output, bool is_analyze,
-                         bool is_show_cmd,
                          ulonglong query_time_in_progress_ms= 0);
 
   /* If true, at least part of EXPLAIN can be printed */
@@ -512,6 +529,10 @@ public:
 
   Explain_update *get_upd_del_plan() { return upd_del_plan; }
 private:
+  bool print_query_blocks_json(Json_writer *writer, const bool is_analyze);
+  void print_query_optimization_json(Json_writer *writer);
+  void send_explain_json_to_output(Json_writer *writer, select_result_sink *output);
+ 
   /* Explain_delete inherits from Explain_update */
   Explain_update *upd_del_plan;
 
@@ -533,6 +554,8 @@ private:
 #ifndef DBUG_OFF
   bool can_print_json= false;
 #endif
+
+  Exec_time_tracker optimization_time_tracker;
 };
 
 
@@ -750,6 +773,8 @@ public:
   Explain_table_access(MEM_ROOT *root, bool timed) :
     derived_select_number(0),
     non_merged_sjm_number(0),
+    cost(0.0),
+    loops(0.0),
     extra_tags(root),
     range_checked_fer(NULL),
     full_scan_on_null_key(false),
@@ -819,6 +844,10 @@ public:
   ha_rows rows;
   double filtered;
 
+  /* Total cost incurred during one execution of this select */
+  double cost;
+
+  double loops;
   /* 
     Contents of the 'Extra' column. Some are converted into strings, some have
     parameters, values for which are stored below.
@@ -908,15 +937,14 @@ public:
                     uint select_id, const char *select_type,
                     bool using_temporary, bool using_filesort);
   void print_explain_json(Explain_query *query, Json_writer *writer,
-                          bool is_analyze, bool no_tmp_tbl);
+                          bool is_analyze);
 
 private:
   void append_tag_name(String *str, enum explain_extra_tag tag);
   void fill_key_str(String *key_str, bool is_json) const;
   void fill_key_len_str(String *key_len_str, bool is_json) const;
   double get_r_filtered();
-  void tag_to_json(Json_writer *writer, enum explain_extra_tag tag,
-                   bool no_tmp_tbl);
+  void tag_to_json(Json_writer *writer, enum explain_extra_tag tag);
 };
 
 
@@ -940,8 +968,8 @@ public:
     handler_for_stats(NULL)
   {}
 
-  virtual enum explain_node_type get_type() { return EXPLAIN_UPDATE; }
-  virtual uint get_select_id() { return 1; /* always root */ }
+  enum explain_node_type get_type() override { return EXPLAIN_UPDATE; }
+  uint get_select_id() override { return 1; /* always root */ }
 
   const char *select_type;
 
@@ -997,13 +1025,14 @@ public:
   /* TODO: This tracks time to read rows from the table */
   Exec_time_tracker table_tracker;
 
+  Gap_time_tracker extra_time_tracker;
   /* The same as  Explain_table_access::handler_for_stats */
   handler *handler_for_stats;
 
-  virtual int print_explain(Explain_query *query, select_result_sink *output, 
-                            uint8 explain_flags, bool is_analyze);
-  virtual void print_explain_json(Explain_query *query, Json_writer *writer,
-                                  bool is_analyze, bool no_tmp_tbl);
+  int print_explain(Explain_query *query, select_result_sink *output,
+                    uint8 explain_flags, bool is_analyze) override;
+  void print_explain_json(Explain_query *query, Json_writer *writer,
+                          bool is_analyze) override;
 };
 
 
@@ -1023,13 +1052,13 @@ public:
 
   StringBuffer<64> table_name;
 
-  enum explain_node_type get_type() { return EXPLAIN_INSERT; }
-  uint get_select_id() { return 1; /* always root */ }
+  enum explain_node_type get_type() override { return EXPLAIN_INSERT; }
+  uint get_select_id() override { return 1; /* always root */ }
 
-  int print_explain(Explain_query *query, select_result_sink *output, 
-                    uint8 explain_flags, bool is_analyze);
-  void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze, bool no_tmp_tbl);
+  int print_explain(Explain_query *query, select_result_sink *output,
+                    uint8 explain_flags, bool is_analyze) override;
+  void print_explain_json(Explain_query *query, Json_writer *writer,
+                          bool is_analyze) override;
 };
 
 
@@ -1050,14 +1079,36 @@ public:
   */
   bool deleting_all_rows;
 
-  virtual enum explain_node_type get_type() { return EXPLAIN_DELETE; }
-  virtual uint get_select_id() { return 1; /* always root */ }
+  enum explain_node_type get_type() override { return EXPLAIN_DELETE; }
+  uint get_select_id() override { return 1; /* always root */ }
 
-  virtual int print_explain(Explain_query *query, select_result_sink *output, 
-                            uint8 explain_flags, bool is_analyze);
-  virtual void print_explain_json(Explain_query *query, Json_writer *writer,
-                                  bool is_analyze, bool no_tmp_tbl);
+  int print_explain(Explain_query *query, select_result_sink *output, 
+                    uint8 explain_flags, bool is_analyze) override;
+  void print_explain_json(Explain_query *query, Json_writer *writer,
+                          bool is_analyze) override;
 };
 
 
+/*
+  EXPLAIN data structure for subquery materialization.
+
+  All decisions are made at execution time so here we just store the tracker
+  that has all the info.
+*/
+
+class Explain_subq_materialization : public Sql_alloc
+{
+public:
+  Explain_subq_materialization(MEM_ROOT *mem_root)
+    : tracker(mem_root)
+  {}
+
+  Subq_materialization_tracker *get_tracker() { return &tracker; }
+
+  void print_explain_json(Json_writer *writer, bool is_analyze);
+
+private:
+  Subq_materialization_tracker tracker;
+};
+
 #endif //SQL_EXPLAIN_INCLUDED
