@@ -286,7 +286,7 @@ typedef struct st_join_table {
   st_join_table *first_inner;   /**< first inner table for including outerjoin */
   bool           found;         /**< true after all matches or null complement */
   bool           not_null_compl;/**< true before null complement is added      */
-  st_join_table *last_inner;    /**< last table table for embedding outer join */
+  st_join_table *last_inner;    /**< last table for embedding outer join */
   st_join_table *first_upper;  /**< first inner table for embedding outer join */
   st_join_table *first_unmatched; /**< used for optimization purposes only     */
 
@@ -1768,6 +1768,7 @@ public:
   int optimize();
   int optimize_inner();
   int optimize_stage2();
+  int optimize_stage2_and_finish();
   bool build_explain();
   int reinit();
   int init_execution();
@@ -1811,7 +1812,7 @@ public:
     memcpy(dest, src, src_arr.size() * src_arr.element_size());
   }
 
-  /// Overwrites 'ref_ptrs' and remembers the the source as 'current'.
+  /// Overwrites 'ref_ptrs' and remembers the source as 'current'.
   void set_items_ref_array(Ref_ptr_array src_arr)
   {
     copy_ref_ptr_array(ref_ptrs, src_arr);
@@ -2712,6 +2713,8 @@ void propagate_new_equalities(THD *thd, Item *cond,
 
 #define PREV_BITS(type, N_BITS) ((type)my_set_bits(N_BITS))
 
+double estimate_post_group_cardinality(JOIN *join, double join_output_card);
+
 bool dbug_user_var_equals_str(THD *thd, const char *name, const char *value);
 
 #endif /* SQL_SELECT_INCLUDED */
